#ifndef Observable_h
#define Observable_h

#include <stdlib.h>
#include <iostream>
#include <math.h>
#include <string>
#include <map>
#include <list>

#include "protein.h"
#include "pool.h"

// forward declaration of Protein
class Protein;

// Laziness: maximum length of a pool or protein label in the setup file
#define MAX_POOLNAME_LENGTH 128

//! Class for system-wide observables.

//! Implementation of system-wide observables (= query functions), which are calculated from 
//! contents of the pools, internal states of proteins, etc.
class Observable
{
	public:
	
	//! Default constructor; creates a new observable with a name
	Observable(const std::string name);
	//! Destructor
	~Observable();
	
	//! @name Setup functions
	//@{
	//! Tell the observable about the pointer to the map of external pools 
	//! and the existing proteins defined during the simulation setup. 
	void setPools(std::map<std::string, Pool*>* mapOfPools, std::list<Protein * >* listOfProteins);
	//! Get pointer to a certain external pool
	Pool* getPool(std::string poolName);
	//! Read parameters (pool names, rate constants, etc) from setup line.
	//! Virtual, because this is specific for each type.
	virtual bool parseLine(char *line) { };
	//@}

	
	//! Calculate and return the value of the observable.  
	//! This is the central function of the observable.
	//! Made virtual so that it has to be implemented in each derived class.
	virtual double get_value() { };
	
	//! get_value() from specific protein with index idx 
	//! The relevant type is hard coded in the respective observable, just pick a certain copy.
	virtual double get_value(const long int &idx) { };
	
	//! Formatted output for some cases
	virtual std::string get_formattedValue();

	
	protected:
	
	//! Check whether all neccessary pools exist in the simulation's list of pools.
	//! This functions directly aborts the simulation (with exit()) when any pool is missing --
	//! there is no way to correct such a setup error later in the simulation.
	virtual void verify_pools() { };
	
	//! String label for this observable
	std::string myName;
	
	//! pointer to map of <name of Pool, pointer to Pool> (this map is part of class Simulation)
	std::map<std::string, Pool*>* Pools;

	//! pointer to the list of proteins
	std::list<Protein*>* Proteins;
};


#endif /* Observable_h */
