// Definitions for the automated inclusion into the parser
// LABEL:		DeltaPhi_Cap
// CLASSNAME:	ObservableDeltaPhiCapacitor
// 
#ifndef OBSERVABLEDELTAPHICAPACITOR_H
#define OBSERVABLEDELTAPHICAPACITOR_H

#include "observable.h"
#include "parameter.h"
#include <math.h>


//! Observable for the transmembrane potential of a spherical capacitor.

//! Subclass representing the transmembrane potential deltaPhi,
//! derived from Observable.
//!
//! Parameters are:
//! - r_i, the inner vesicle radius
//! - d, the membrane thickness
//! - epsilon, the relative eps of the membrane
//!
//! DeltaPhi accesses three pools: for the free protons inside the vesicle, 
//! for those bound to the membrane and for those created from internal charge 
//! separations in the proteins.
//!
//! The underlying physical model is a spherical capacitor with different 
//! dielectric constants. However, as only the voltage across the membrane 
//! is calculated, the epsilon outside of the membrane is not important, 
//! only the dielectric constant of the membrane itself.

class ObservableDeltaPhiCapacitor : public Observable
{
	public:
	
	//! Constructor
	ObservableDeltaPhiCapacitor(const std::string name);
	//! Destructor
	~ObservableDeltaPhiCapacitor();
	
	//! Calculate and return the value of the transmembrane potential. 
	//DeltaPhi is returned in units of milliVolts [mV]. 
	double get_value();
	
	//! Parser for line from input file
	bool parseLine(char *linep);
	
	
	private:
	
	//! @name Char[]s for the pool names (see length above)
	//@{
	char FreeHPool[MAX_POOLNAME_LENGTH], 
		 MembraneHPool[MAX_POOLNAME_LENGTH], 
		 ProteinHPool[MAX_POOLNAME_LENGTH],
		 c2redPool[MAX_POOLNAME_LENGTH],
		 c2oxPool[MAX_POOLNAME_LENGTH];
	//@}
	 
	//! name pointer to pools and variables for reaction constants to 
	//! avoid their continuous lookup
	//@{
	Pool *FreeHPoolp, *MembraneHPoolp, *ProteinHPoolp, *c2redPoolp, *c2oxPoolp;
	double r_a, d_M, r_i, eps, Q_offset;
	double freeHScaling, membraneHScaling, proteinHScaling, c2redScaling, c2oxScaling;
	//@}
	
	//! verify that all neccessary pools are defined
	void verify_pools();

};
#endif /* OBSERVABLEDELTAPHICAPACITOR_H */
