// Definitions for the automated inclusion into the parser
// LABEL:		DeltaPhi_ChemCap2
// CLASSNAME:	ObservableDeltaPhiChemCapacitor2
// 
#ifndef ObservableDeltaPhiChemCapacitor2_h
#define ObservableDeltaPhiChemCapacitor2_h

#include "observable.h"
#include "parameter.h"
#include <math.h>


//! Observable for the transmembrane potential of a spherical capacitor plus the chemical contribution, V2.

//! Parameters are:
//! - the voltage per unit charge inside the vesicle [mV / e]
//! - the voltage per pH unit across the membrane    [mV / Delta pH]
//!
//! The chemical contribution is calculated via the Nernst relation from
//! the pH difference between inside and outside of the vesicle.
//!
//! DeltaPhi accesses five pools of charged species: 
//! * for the free protons inside the vesicle, 
//! * for protons bound to the membrane,
//! * for charges  created from internal charge separations in the proteins,
//! * and for difference between numbers in the two cytochrome c pools
//!
//! Additionally, for the pH gradient, the pools of protons outside of the vesicle is accessed.
//!
//! DeltaPhi is given in units of milliVolts [mV].

class ObservableDeltaPhiChemCapacitor2 : public Observable
{
	public:
	
	//! Constructor
	ObservableDeltaPhiChemCapacitor2(const std::string name);
	//! Destructor
	~ObservableDeltaPhiChemCapacitor2();
	
	//! Calculate and return the value of the transmembrane potential (in units of milliVolts [mV])
	double get_value();
	
	//! Parser for line from input file
	bool parseLine(char *linep);
	
	
	
	private:
	
	//! @name Char[]s for the pool names (see length above)
	//@{
	char HoutsidePool[MAX_POOLNAME_LENGTH], 
		 FreeHPool[MAX_POOLNAME_LENGTH], 
		 MembraneHPool[MAX_POOLNAME_LENGTH], 
		 ProteinHPool[MAX_POOLNAME_LENGTH],
		 c2redPool[MAX_POOLNAME_LENGTH],
		 c2oxPool[MAX_POOLNAME_LENGTH];
	//@}
	 
	//! name pointer to pools and variables for reaction constants to 
	//! avoid their continuous lookup
	//@{
	Pool *HoutsidePoolp, *FreeHPoolp, *MembraneHPoolp, *ProteinHPoolp, *c2redPoolp, *c2oxPoolp;
	double pHVoltage, voltagePerCharge, Q_offset;
	double freeHScaling, membraneHScaling, proteinHScaling, c2redScaling, c2oxScaling;
	//@}
	
	//! verify that all neccessary pools are defined
	void verify_pools();

};
#endif /* ObservableDeltaPhiChemCapacitor_h */
