//	ATPase: takes Protons out from the inside of the vesicle and 
//	converts ADP to ATP with 1ATP for 4 protons

// #define DEBUG

#include"proteinATPase.h"

using namespace std;


//	Rate for ATP-output of the ATPase, fit to experiment
//	=> proton processing rate is 4 times higher
double rate(double dphi)
{
	return (0.25* (380.0 * ( 1.0/( 1.0 + (pow (10.0, (0.008*0.008*0.008*(160.0*160.0*160.0 - dphi*dphi*dphi)))))) - 2.97));
}


ProteinATPase::ProteinATPase(const std::string name, const double &timestep) :
Protein::Protein(name, timestep)
{
	// init variables
	reg_H = 0;
	bs_Hin = false;
	
	// pointer to function declarations:
	function fu1;
	void (Protein::*pr1) (Protein *) = (void(Protein::*)(Protein *))(&ProteinATPase::reaction1a);
	fu1.pfunction = pr1;
	setFunction("r1a", fu1);
	
	void (Protein::*pr2) (Protein *) = (void(Protein::*)(Protein *))(&ProteinATPase::reaction1b);
	fu1.pfunction = pr2;
	setFunction("r1b", fu1);

}


std::string ProteinATPase::listInternals() {
	return(string("# time\treg_H\tbs_Hin"));
}


void ProteinATPase::writeInternals()
{
	if(outputInternals)
	{
		double realTime = timer * delta_t;
																   
		internalsOutFile.open(internalFileName.c_str(), fstream::app);
		internalsOutFile << realTime << "\t" << reg_H << "\t" << bs_Hin << "\n";
		internalsOutFile.close();
	}
}

void ProteinATPase::verify_pools()
{
	if(!Pools->count(string(HinsidePool))) {
		cerr << "ProteinATPase::verify_pools: pool "<<HinsidePool<<" not defined" << endl;
		exit(-3);
	}
	if(!Pools->count(string(HoutsidePool))) {
		cerr << "ProteinATPase::verify_pools: pool "<<HoutsidePool<<" not defined" << endl;
		exit(-3);
	}
	if(!Pools->count(string(ADPPool))) {
		cerr << "ProteinATPase::verify_pools: pool "<<ADPPool<<" not defined" << endl;
		exit(-3);
	}
	if(!Pools->count(string(ATPPool))) {
		cerr << "ProteinATPase::verify_pools: pool "<<ATPPool<<" not defined" << endl;
		exit(-3);
	}
	
	if(!(Observables->count(string(DeltaPhiObs)))) {
		cerr << "ProteinATPase::verify_pools: observable "<<DeltaPhiObs<<" not defined" << endl;
		exit(-3);
	}
	
	HinsidePoolp = getPool(HinsidePool);
	HoutsidePoolp = getPool(HoutsidePool);
	ADPPoolp = getPool(ADPPool);
	ATPPoolp = getPool(ATPPool);
	
	DeltaPhiObsp = getObservable(DeltaPhiObs);
}


// Currently, the rate constants are not used at all...
bool ProteinATPase::parseLine(char *linep)
{
	double rate0, rate1, rate2, rate3;
	char  dummy[128];
	int num_of_proteins;

	if(sscanf(linep, "%s %d %s %lf %s %lf %s %lf %s %lf %s", 
		dummy, &num_of_proteins, ADPPool, &rate0, ATPPool, &rate1, HinsidePool, &rate2, HoutsidePool, &rate3, DeltaPhiObs) != 11) {
		cerr << "Simulation::parseFile: not enough parameters for ATPase line >>" << linep << "<<" << endl;
		return(false);
	}
	// Sanity check on the (yet unused) rates
	if((rate0<0) || (rate1<0) || (rate2<0) || (rate3<0)) {
		cerr << "Simulation::parseFile: negative rate constant in ATPase line >>" << linep << "<<" << endl;
		return(false);
	}
	
	#ifdef SCHWAETZ
	cout << "------------------" << endl;
	cout << "   ATPase: read: " << num_of_proteins << endl;
	cout << "   ATPase: read: " << ADPPool << "  " << rate0 << endl;
	cout << "   ATPase: read: " << ATPPool << "  " << rate1 << endl;
	cout << "   ATPase: read: " << HinsidePool << "  " << rate2 << endl;
	cout << "   ATPase: read: " << HoutsidePool << "  " << rate3 << endl;
	cout << "   ATPase: read: " << DeltaPhiObs << endl;
	#endif

	return(true);
}


// ++++++++++++++ internal reactions ++++++++++++++++++++++++


// pocess bound H+ according to dPhi-dependent rate
void ProteinATPase::reaction1a(Protein *p)
{
	// check, if there is a proton bound and 1 ADP outside
	if (bs_Hin && (ADPPoolp->getInside() >= 1))
	{
		// read deltaPhi from global observable
		double dphi = DeltaPhiObsp->get_value();
		// random number
		double probability = lin_ran();
		// 4*rate, as only one H+ is taken up
		double P = delta_t * 4.0 * rate(dphi);
		
		if ((probability <= P))
		{
			bs_Hin = false;
			HoutsidePoolp->put_back();
			reg_H++;
			
			// check for 4 protons => release one ATP
			if ((reg_H >= 4))
			{
				reg_H -= 4;
				
				ADPPoolp->take(1);
				ATPPoolp->put_back();
				
				#ifdef DEBUG
				cout << "ATPase: released one ATP, protons = " << reg_H << endl;
				#endif
			}
			writeInternals();
		}
		
		#ifdef DEBUG
		cerr << "ProteinATPase::reaction1b: DeltaPhi = " << dphi << endl;
		#endif
	}
}


// bind a proton from the inside, if one is available
void ProteinATPase::reaction1b(Protein *p)
{
	#ifdef DEBUG
	cerr << "ProteinATPase::reaction1b: start" << endl;
	#endif
	
	if(!bs_Hin) {
		if (HinsidePoolp->take(1))
		{
			bs_Hin = true;
			writeInternals();
		}
		
		#ifdef DEBUG
		cerr << "ProteinATPase::reaction1b: done" << endl;
		#endif
	}
}
