// Definitions for the automated inclusion into the parser
// LABEL:		bc1dimer
// CLASSNAME:	ProteinBC1Dimer
// 
#ifndef ProteinBC1Dimer_h
#define ProteinBC1Dimer_h


#include<string>
#include<stdlib.h>
#include<iostream>
#include<string>

#include "random.h"
#include "protein.h"
#include "pool.h"
#include "parameter.h"


//! Class that represents bc1 protein complex of the vesicle.

/*!
This class represents the cytochrome bc1 complex in its dimeric form, which transfers Protons to the inside of the vesicle. 
It takes the 2 electron proton pairs from a quinol and transfers both protons to the inside. 
One electron is passed via the Rieske (iron-sulphur, FeS) protein  to a waiting cytochrome c2 on the c2 binding site. 
The other electron and a proton from the outside are transfered to a (semi)quinone on the Qi site.
The emptied Q can directly pass from the Qo to the Qi site with a given probability.
Binding sites: Q0 to dock the Quinol, Qi to dock the Quinone, c2 to dock the cytochrome c2.
Internal Values: Qi, Q0 for the reduction state of Quinone and Quinol 
FeS for an electron on the FeS protein.
Pools: Q-Pool, QH2-Pool, Cytochrome c2 oxidated, Cytochrome c2 reduced, H+Outside, H+Inside

Order of parameter in the setup file (7*(pool rate), 1*pool, 4*rate):
QH2onPool		QH2.kon
QoffPool		Q.koff
QonPool			Q.kon
QH2offPool		QH2.koff
c2oxPool		c2ox.kon
c2redPool		c2red.koff
HoutsidePool	H+outside.kon
HinsidePool		--
--				k.QoQiSwap
--				k.eQoFeS
--				k.HQi
--				k.FeSSwap
--				k.FeSSwap.phi0
*/

class ProteinBC1Dimer : public Protein
{
	public:
	
	//! Constructor
	ProteinBC1Dimer(const std::string name, const double &timestep);
	//! Destructor
	~ProteinBC1Dimer();
	
	//! Parse line from setup file
	bool parseLine(char *linep);
	
	
	private:
	
	//! @name internal functions (reactions) of the protein
	//@{
	//! binds a QH2 molecule at either of the Qo sites
	void reaction1(Protein *);
	//! binds a Q molecule at the Qi sites
	void reaction2(Protein *);
	//! binds a cytochrome c2 at FeS (= c1) site, only one c2 can be bound at a time
	void reaction3(Protein *);
	//! bind proton from outside the vesicle
	void reaction4(Protein *);
	//! unbinding of emptied Q from Q0 site and possible rebinding at Qi
	void reaction5(Protein *);
	//! two separate reactions for unbinding of the QH2 from Qi site 
	void reaction6_1(Protein *);
	void reaction6_2(Protein *);
	//! two separate reactions for unbinding of c2 from c2 site
	void reaction7_1(Protein *);
	void reaction7_2(Protein *);
	//! electron transfer from Qo -> FeS + Qi plus release of the two protons into the vesicle
	void reaction8(Protein *);
	//! fill up the proton on the Qi
	void reaction9(Protein *p);
	// Movement of the FeS domain and electron transfer onto the bound c2
	void reaction10_1(Protein *p);
	void reaction10_2(Protein *p);
	//@}
	
	//! Defs of the internal registers and binding sites.
	//! Two copies each, it's a dimer...
	//! (registers = int,  binding sites = bool)
	// @{
	//! Qo Binding site for quinol QH2 -> Q
	bool bs_Qo_1, bs_Qo_2;
	//! Qi Binding site for quinone Q -> QH2
	bool bs_Qi_1, bs_Qi_2;
	//! cytochrome c2 binding site at the Rieske protein
	bool bs_c2_1, bs_c2_2;
	//! bs for protons from outside
	bool bs_H_1, bs_H_2;
	//! number of e-H-pairs still to unload on the Qo Q
	int reg_Qo_1, reg_Qo_2;
	//! number of electrons on the Qi Q
	int reg_Qie_1, reg_Qie_2;
	//! number of protons on the Qi Q
	int reg_QiH_1, reg_QiH_2;
	//! electron on the FeS subunit
	int reg_FeS_1, reg_FeS_2;
	//! electron on the c2
	int reg_c2_1, reg_c2_2;
	// @}
	
	//! @name Char[]s for the pool and observable names (see length above)
	//! These pools are the same for both halves of the dimer.
	//@{
	char QoffPool[MAX_POOLNAME_LENGTH], 
		 QH2offPool[MAX_POOLNAME_LENGTH], 
		 QonPool[MAX_POOLNAME_LENGTH], 
		 QH2onPool[MAX_POOLNAME_LENGTH], 
		 c2redPool[MAX_POOLNAME_LENGTH], 
		 c2oxPool[MAX_POOLNAME_LENGTH], 
		 HoutsidePool[MAX_POOLNAME_LENGTH], 
		 HinsidePool[MAX_POOLNAME_LENGTH];
	char DeltaPhiObs[MAX_POOLNAME_LENGTH];
	//@}

	//! Pointers to pools and variables for reaction constants to avoid their repeated lookup
	//@{
	Pool *QoffPoolp, *QH2offPoolp, *QonPoolp, *QH2onPoolp, *c2redPoolp, *c2oxPoolp, *HoutsidePoolp, *HinsidePoolp;
	Observable *DeltaPhiObsp;
	double QH2_kon, Q_kon, c2ox_kon, Houtside_kon, k_QoQiSwap, Q_koff, QH2_koff, c2red_koff, k_eQoFeS, k_eQoFeS_phi0, k_HQi, k_FeSSwap;
	//@}
	
	//! timer variables -- faster than the repeated lookup in the function structure
	unsigned long int timer_r6_1, timer_r6_2, timer_r7_1, timer_r7_2, timer_r10_1, timer_r10_2;

	//! verify that all neccessary pools are defined
	void verify_pools();

	//! @name Listing the internals
	// @{
	std::string listInternals();
	void writeInternals();
	// @}
};
#endif
