#include "simulation.h"

using namespace std;


// #define DEBUG
// #define VERBOSE


// Constructor for simulation with some reasonable defaults
Simulation::Simulation()
: num_of_steps(0),
logstep(1),
Proteins(),
Pools(),
PoolNames(),
Observables(),
ObservableNames(),
output_file(""),
output_BS(""),
output_console("none"),
output_progress("")
{
}


// Destructor
Simulation::~Simulation()
{
}



list<Protein*>::iterator Simulation::getNextPosition()
{
	int cnt = 0;
	list<Protein*>::iterator it = Proteins.begin();
	unsigned long int lang = Proteins.size();
	unsigned long int pos = 0;
	if(lang > 1) {
		pos = int_ran(lang);
	}
	for(unsigned long int i=0; i<pos; i++, cnt++) {
		it++;
	}
	return(it);
}


//! Parser for simulation setup file.
//! There are 5 kinds of line types in such a file.
//! Empty lines are allowed, Proteins have to be declared before the Pools.
//!
//! - protein: <protein type> <index of this protein> |: [pool name] [rate] :|
//! - observable: <name> <subtype> <list in output> <parameter1> ...
//! - pool: <name> <infinite> <initial number of particles in pool> <volume> <show_in_output?>
//! - change: <poolName> <time> <new density>
//! - # comment
//!
//! Any text on a line after the required data is ignored and may be used for comments
//!
//! Beware of length limits: a setup line is silently truncated to 2048 chars, 
//! and protein/pool/observable names have to be <=127 chars. Otherwise unexpected things may occur.
//!
//! Any error during the setup simply terminates the parser, as there is no sense in trying to
//! guess correct values for incorrectly specified data.
//
// @todo Check for length limits in the setup file

void Simulation::parseFile(std::string filename)
{
	ifstream file_in;
	file_in.open(filename.c_str(), fstream::in);

	#ifdef VERBOSE
	cout << "Simulation::parseFile: starting" << endl;
	cout << "Simulation::parseFile: output_BS = " << output_BS << endl;
	#endif
	
	// initialize variables to which read data is passed
	char newLine[MAX_INPUTLINE_LENGTH], line[MAX_INPUTLINE_LENGTH], 
		 mode[MAX_PROTEINNAME_LENGTH], Pool_infinite[MAX_PROTEINNAME_LENGTH], 
		 Pool_Name[MAX_PROTEINNAME_LENGTH], protein_count[MAX_PROTEINNAME_LENGTH], 
		 list_Pool[MAX_PROTEINNAME_LENGTH];
	char dummy[MAX_PROTEINNAME_LENGTH], subtype[MAX_PROTEINNAME_LENGTH];
	char mode_trunc10[11];
	int V_volume, V_surface, num_of_proteins, Pool_Particles;
	double Pool_Volume, rate0, rate1, rate2, rate3, rate4, rate5, rate6, rate7, rate8, rate9, rate10, rate11, Light_Time;
	light_periode actual_light;
	char pool0[MAX_PROTEINNAME_LENGTH], pool1[MAX_PROTEINNAME_LENGTH], 
		 pool2[MAX_PROTEINNAME_LENGTH], pool3[MAX_PROTEINNAME_LENGTH], 
		 pool4[MAX_PROTEINNAME_LENGTH], pool5[MAX_PROTEINNAME_LENGTH], 
		 pool6[MAX_PROTEINNAME_LENGTH], pool7[MAX_PROTEINNAME_LENGTH], 
		 pool8[MAX_PROTEINNAME_LENGTH];
	double last_Light_Time = -1.0;
	
	bool stopper = true;
	while(stopper)
	{ 
		// read complete line
		file_in.getline(newLine, MAX_INPUTLINE_LENGTH);
		
		#ifdef DEBUG
		cout << "parser: next line: >>" << newLine << "<<" << endl;
		#endif
		
		// check for comments
		if (!file_in.eof())
		{
			if((newLine[0] == '#') || (strlen(newLine) == 0))
			{
				#ifdef DEBUG
				cout << "parser: found comment: " << newLine << "\n";
				#endif
				
				mode[0] = '\0';
				line[0] = '\0';
			}
			else
			{
				// extract first word
				int i = 0;
				int k = 0;
				int lang = strlen(newLine);
				while((newLine[i] != ':') && (i<lang))
				{
					// truncate keyword if longer than 127 chars
					if(k < (MAX_PROTEINNAME_LENGTH-1)) {
						mode[i] = newLine[i];
					}
					i++;
					k++;
				}
				mode[i] = '\0';
				
				// copy rest of the line into "line"
				// newLine was already limited to 2048 chars => no further length check
				if(i<lang) {
					strcpy(line, &newLine[i+1]);
				} else {
					line[0] = '\0';
				}
			}
			
			#ifdef DEBUG
			cout << "getline: mode = >>" << mode << "<<" << endl;
			cout << "getline: line = >>" << line << "<<" << endl;
			#endif
			
			
			// check which information the line contains: protein, observable, pool or concentration change
			// ########################################################################
			// this is data of a protein
			if (strcmp(mode, "protein") == 0)
			{
				int laufende_nummer;
				sscanf(line, "%s %d", mode, &laufende_nummer);
				sprintf(protein_count, "_%d", laufende_nummer);
				
				// create a shortened label to allow for multiple independent connectors
				// @todo Find a more general way to create shorter labels of any length
				strncpy(mode_trunc10, mode, 10);
				mode_trunc10[9] = '\0';
				
				#include "myProteinParser.cpp"
				
				{
					cerr << "unknown protein " << mode << endl;
				}
				
			} 
			// ########################################################################
			// this is data for a system wide observable
			else if (strcmp(mode, "observable") == 0)
			{
				Observable *obs = NULL;
				char obs_name[] = "aVeryVeryLongDummyTextThatShouldNotBeTooShortOrWeRunIntoTroublesWithProbablyWeirdResultsOrAtLeastACrash";
				int laenge = strlen(obs_name) - 1;
				
				sscanf(line, "%s ", mode);
				
				#include "myObservableParser.cpp"
				
				{
					cerr << "Parser: unknown observable " << mode << endl;
				}
				
				if(obs)
				{
					// parse data line
					if ((obs->parseLine(line)) == false) {
						exit(-2);
					}
					
					// insert into map of observables
					Observables[obs_name] = obs;
					
					// if to be listed, append to list of observable names
					sscanf(line, "%s %s ", mode, list_Pool);
					if(strcmp(list_Pool, "true") == 0) {
						ObservableNames.push_back(obs_name);
					}
				}
			}
			// ########################################################################
			// this is data for a pool
			else if (strcmp(mode, "pool") == 0)
			{
				bool infinite_value = false;	// defaults: finite pool, output as particles
				string list_value = "part";	
				
				// read date into the variables
				if(sscanf(line, "%s %s %d %lf %s", Pool_Name, Pool_infinite, &Pool_Particles, &Pool_Volume, list_Pool) != 5) {
					cerr << "Simulation::parseFile: not enough parameters for pool line >>" << line << "<<" << endl;
					exit(-2);
				}
				// weitere Fehler abfangen
				if((Pool_Particles < 0) || (Pool_Volume < 0.0)) {
					cerr << "Simulation::parseFile: negative pool size or number of particles < 0 in pool line " << line << endl;
					exit(-2);
				} 
				
				#ifdef DEBUG
				cout << "read data: " << Pool_Name << " " << Pool_infinite << " " << Pool_Particles << " " << Pool_Volume << "\n";
				#endif 
				
				#ifdef VERBOSE
				cout << "Pool: " << Pool_Name << " " << Pool_infinite << " " << Pool_Particles << " " << Pool_Volume << "\n";
				#endif
				
				// Check infinity of the pool
				if (strcmp(Pool_infinite, "true") == 0) {
					infinite_value = true;
				}
				else if (strcmp(Pool_infinite, "false") == 0) {
					infinite_value = false;
				}
				else {
					cerr << "Simulation::parseFile(std::string filename) -- " << Pool_Name << ": ambigious infinite parameter, assuming infinite = false\n";
					infinite_value = false;
				}
				
				
				// (How) Is the pool to be listed in the output?
				if (strcmp(list_Pool, "part") == 0) {
					list_value = "part";
				}
				else if (strcmp(list_Pool, "conc") == 0) {
					list_value = "conc";
				}
				else if (strcmp(list_Pool, "false") == 0) {
					list_value = "false";
				}
				else {
					cerr << "Simulation::parseFile: >>" << list_Pool << "<< is not a valid flag for pool listing, assuming listed = \"part\" in pool line " << line << endl;
					list_value = "part";
				}
				
				// create a new Pool* in the Pools map
				Pools[Pool_Name] = new Pool(Pool_Particles, Pool_Volume, infinite_value, list_value, delta_t);
				// add the pool's name to the list of the pool names
				PoolNames.push_back(Pool_Name);
			}
			
			// ########################################################################
			// read concentration changes
			else if (strcmp(mode, "change") == 0)
			{
				// read information
				double Light_Intensity;		// for historic reasons the changing pools are still labelled as "light" pool :-)
				if(sscanf(line, " %s %lf %lf", pool0, &Light_Time, &Light_Intensity) != 3) {
					cerr << "Simulation::parseFile: not enough parameters for change line >>" << line << "<<" << endl;
					exit(-2);
				}
				
				// Catch false input:
				// Does the pool exist?
				if(Pools.count(pool0) == 0) {
					cerr << "Simulation::parseFile: unknown pool "<< pool0 << " at changes line >>" << line << "<<" << endl;
					exit(-2);
				}
				// Are the changes sorted in time?
				if(Light_Time < last_Light_Time) {
					cerr << "Simulation::parseFile: new change time not later than previous time at change line >>" << line << "<<" << endl;
					exit(-2);
				}
				// Do not allow negative concentrations 
				// !!! Please, never change this one. !!!
				// If you need a negative concentration, then add an observable that multiplies a value by (-1) and use that
				if(Light_Intensity < 0.0) {
					cerr << "Simulation::parseFile: negative concentration at change line >>" << line << "<<" << endl;
					exit(-2);
				}
				
				double poolVol = Pools[pool0]->getVolume();
				
				#ifdef VERBOSE
				cout << "change: " << line << endl;
				cout << "  pool = " << pool0 << "   time = " << Light_Time << "  concentration = " << Light_Intensity << endl;
				cout << "  Volume of pool " << pool0 << " = " << poolVol << endl;
				#endif /* VERBOSE  */
				
				// append name of the pool to list of changing pools (and delete duplicates)
				ChangesNames[pool0] = poolVol;
								
				// create new light_periode struct:
				actual_light.poolName = string(pool0);
				actual_light.time = Light_Time;
				actual_light.particles = static_cast<long int>(Light_Intensity * poolVol + 0.5);
				actual_light.infinite = true;
				
				// add this object to the list of changes:
				Light.push_back(actual_light);
				last_Light_Time = Light_Time;
			}
		}
		
		// ########################################################################
		else
		{
			// MAKE sure that EOF is reached
			if (file_in.eof())
			{
				#ifdef DEBUG
				cout << "end of file\n";
				#endif
				// no more lines will be read
				stopper = false;
				
				// Finalize setup: connect pools, proteins, observables
				// 
				// create pointer to map<string, Pool*>
				std::map<std::string, Pool*>* ptrToPool = &Pools;
				// create pointer to map<string, Observable*>
				std::map<std::string, Observable*>* ptrToObservable = &Observables;
				// create pointer to Proteins list
				std::list<Protein*>* ptrToProtein = &Proteins;
				
				// pass pointers to Pools and Observables to all Proteins
				list<Protein*>::iterator iter;
				for ( iter = Proteins.begin(); iter != Proteins.end() ; iter++) {
					(*iter)->setPoolsObservables(ptrToPool, ptrToObservable);
				}
				
				// pass pointer to pools and proteins to observables
				map<std::string, Observable*>::iterator itero;
				for ( itero = Observables.begin(); itero != Observables.end() ; itero++) {
					(itero->second)->setPools(ptrToPool, ptrToProtein);
				}

				
				// Initialize output of the internals, if desired
				if(output_BS != "")
				{
					#ifdef VERBOSE
					cout << "------------------" << endl;
					#endif /* VERBOSE */
					for (iter = Proteins.begin(); iter != Proteins.end() ; iter++)
					{
						string ausgabeDatei = output_BS + "_" + (*iter)->readMyName() + string("_internals.dat");
						(*iter)->initInternalOutput(ausgabeDatei);
					}
				}
			}
		}
	} // end of the central parser loop, uff.
	
	// close setup file
	file_in.close();
	
	#ifdef VERBOSE
	cout << "Proteins: " << Proteins.size() << "\n";
	cout << "Pools:    " << Pools.size() << "\n";
	#endif /* VERBOSE */
}


void Simulation::run()
{
	#ifdef VERBOSE
	cout << "simulation started\n";
	cout << "delta_t = " << delta_t << endl;

	// List the proteins in the list
	{
		cout << "Protein list:";
		list<Protein*>::iterator iter;
		for (iter = Proteins.begin(); iter != Proteins.end() ; iter++) {
			cout << "   " << (*iter)->readMyName();
		}
		cout << endl;
	}
	#endif

	// create file streams for output files
	ofstream file_out_1;
	#ifdef HAVE_PROGRESS_FILE
	ofstream file_progress;
	#endif
	
	// open output files if desired
	if (output_file != "")
		{
		file_out_1.open(output_file.c_str(), fstream::out);
		
		file_out_1 << "#time";
		list<string>::iterator iterNames;
		for(iterNames = PoolNames.begin(); iterNames != PoolNames.end(); ++iterNames) {
			if(Pools[(*iterNames)]->isListed()) {
				file_out_1 << "  \t" << *iterNames;
			}
		}
		for(iterNames = ObservableNames.begin(); iterNames != ObservableNames.end(); ++iterNames) {
			file_out_1 << "  \t" << *iterNames;
		}
		file_out_1 << endl;
	}
	
	// Check for meaningful output to stdout
	if (output_console != "none" && output_console != "particles")
	{
		cerr << "Simulation::run(): unspecified console output; assuming: \"none\"\n";
		output_console = "none";
	}
	
	if (output_console == "particles")
	{
		cout << "#time";
		list<string>::iterator iterNames;
		for(iterNames = PoolNames.begin(); iterNames != PoolNames.end(); ++iterNames) {
			if(Pools[(*iterNames)]->isListed()) {
				cout << "  \t" << *iterNames;
			}
		}
		for(iterNames = ObservableNames.begin(); iterNames != ObservableNames.end(); ++iterNames) {
			cout << "  \t" << *iterNames;
		}
		cout << endl;
	}
	
	
	// setup for ASCII art progress bar
	unsigned long int hash_steps = num_of_steps + 10;
	unsigned long int hash_counter = 0;
	if (output_console == "none") {
		// check COLUMNS environ variable
		char colName[] = "COLUMNS";
		char *cols = getenv(colName);
		// if undef, default to 80
		int spalten = 80;
		if(cols != 0) {
			if(sscanf(cols, "%d", &spalten) != 1) {
				spalten = 80;
			}
		}
		hash_steps = static_cast<unsigned long int>(num_of_steps / (spalten-2));
		
		int i;
		for (i=0; i<(spalten-1); ++i) {
			printf("-");
			fflush(stdout);
		}
		printf("|\r|");
		fflush(stdout);

	}
	
	#ifdef HAVE_PROGRESS_FILE
	// setup percentage output
	unsigned long int percent_steps = num_of_steps / 50;
	if (output_progress != "") {
		file_progress.open(output_progress.c_str(), fstream::out);
		file_progress << 0 << endl;
		file_progress.close();
	}
	#endif /* HAVE_PROGRESS_FILE */
	

	// Time of next change of pool concentrations
	unsigned long int time2switch = static_cast<unsigned long int>(Light.front().time / delta_t + 0.5);
	
	// #######################################################################
	// Setup and preflight complete: Here's the main simulation loop (Taraa!!!)
	// => iterate over Proteins and do Protein.timestep()
	for (unsigned long int i = 0; i <= num_of_steps; i++)
	{
		// switch pool concentrations?
		while ((i >= time2switch) && !Light.empty() )
		{
			// get data from list and apply
			light_periode actual_light = Light.front();
			Pools[actual_light.poolName]->setInside(actual_light.particles);
			// remove first element from list, we don't need it anymore
			Light.pop_front();
			// get next a'changin' time
			if(!Light.empty()) {
				time2switch = static_cast<unsigned long int>(Light.front().time / delta_t + 0.5);
			} else {
				time2switch = num_of_steps+1;  // proceed to the end of the universe and order a drink
			}
		}

		
		// -----------------------------------------------------------------------
		// trigger internal protein reactions
		list<Protein*>::iterator iter;
		for ( iter = Proteins.begin(); iter != Proteins.end() ; iter++) {
			(*iter)->timestep();
		}
		// -----------------------------------------------------------------------
	
		// gimme some output!
		if ((i % logstep) == 0)
		{  
			// simulation time in seconds
			double time_index = double(i) * delta_t;
			if (output_file != "")
				{
				// write pool level (particles or concentration) into file
				list<string>::iterator iterNames;
				file_out_1 << time_index;
				for(iterNames = PoolNames.begin(); iterNames != PoolNames.end(); ++iterNames) {
					if(Pools[(*iterNames)]->isListed())
					{
						file_out_1 << "\t" << Pools[(*iterNames)]->getOutput();
					}
				}
				// repeat for observables
				for(iterNames = ObservableNames.begin(); iterNames != ObservableNames.end(); ++iterNames) {
					file_out_1 << "\t" << Observables[(*iterNames)]->get_value();
				}
				file_out_1 << endl;
			}
			// do the same to the console
			if (output_console == "particles")
				{
				list<string>::iterator iterNames;
				cout << time_index;
				// first pools
				for(iterNames = PoolNames.begin(); iterNames != PoolNames.end(); ++iterNames) {
					if(Pools[(*iterNames)]->isListed())
					{
						cout << "\t" << Pools[(*iterNames)]->getOutput();
					}
				}
				// then observables
				for(iterNames = ObservableNames.begin(); iterNames != ObservableNames.end(); ++iterNames) {
					cout << "\t" << Observables[(*iterNames)]->get_formattedValue();
				}
				cout << endl;
			}
		}
		
		// any progress?
		if ((i % hash_steps) == 0)
		{
			if(i > 0) {
				cout << "#" << flush;
			}
		}
	
		#ifdef HAVE_PROGRESS_FILE
		if (( i % percent_steps) == 0)
		{
			file_progress.open(output_progress.c_str(), fstream::out);
			file_progress << i/percent_steps << endl;
			file_progress.close();
		}
		#endif /* HAVE_PROGRESS_FILE */

	} 
	
	// time loop done, finalize outputs
	if (output_console == "none") {
		cout << endl;
	}
	file_out_1.close();
	
	#ifdef VERBOSE
	cout << "\nsimulation finished\n\n";
	#endif
}


void Simulation::setNumberOfSteps(unsigned long int num_of_steps_) {
	num_of_steps = num_of_steps_;
}


void Simulation::setLogstep(unsigned long int logstep_) {
	logstep = logstep_;
}


void Simulation::setOutputFilename(std::string output_file_)
{
	#ifdef VERBOSE
	cout << "Simulation::setOutputFilename: set output file to " << output_file_ << endl;
	#endif
	output_file = output_file_;
}


void Simulation::setOutputBS(std::string output_BS_)
{
	#ifdef VERBOSE
	cout << "Simulation::setOutputBS: set output file basename for internals to " << output_BS_ << endl;
	#endif
	output_BS = output_BS_;
}


void Simulation::setOutputConsole(std::string output_console_) {
	output_console = output_console_;
}


void Simulation::setOutputProgress(std::string progress_name) {
	output_progress = progress_name;
}

